require 'sketchup.rb'
require "dl"
require "dl/import"
require "dl/struct"

require Sketchup.find_support_file('midiator.rb', "plugins/SketchyPhysics3/midiator/lib/midiator")

def testDrums()
    lines=File.readlines(Sketchup.find_support_file('Edge1.xml', "plugins/SketchyPhysics3/sounds"))
    score=parseScore(lines)
    playScore(score)
end
def playScore(events)
    mi=midiInterface()
    0.upto(1000){|tick|
        eventArray=events[tick]
        if(eventArray!=nil)
            eventArray.each{|event|
                mi.startNote(event[2],event[3],event[1]-1)    

            }
        end
        sleep(0.030)
    }
end

def testEncode(name)
    lines=File.readlines(Sketchup.find_support_file(name, "plugins/SketchyPhysics3/sounds"))
    score=encodeScore(lines)
end
def encodeScore(lines)
    noteEvents={}
    lines.each_with_index{|l,i|
        if(l.include?("<Absolute>"))
            tick=l.split(/[><]/)[2].to_i
#tick=tick/10
            event=lines[i+1]
            #event='<NoteOn Channel="10" Note="46" Velocity="0"/>'
            #clear out the cruft.
            event.gsub!(/[><\/\"]/,"")
            parts=event.split(" ")
            #puts parts
            if(parts[0]=="NoteOn"|| parts[0]=="NoteOff")
                channel=parts[1].split("=")[1].to_i    
                note=parts[2].split("=")[1].to_i    
                velocity=parts[3].split("=")[1].to_i
                amount=1.0
                amount=0.0 if parts[0]=="NoteOff"
                amount=0.0 if velocity==0
                #noteDesc=(channel<<8)+note
                #noteEvents[noteDesc]={} if noteEvents[noteDesc]==nil
                #noteEvents[noteDesc][tick]=[amount,velocity]                
                
                noteDesc="#{channel.to_s}:#{note.to_s}"
                noteEvents[noteDesc]=[] if(noteEvents[noteDesc]==nil) 
                noteEvents[noteDesc].push([tick,amount,velocity])                
                #puts([tick,parts[0],channel,note,velocity].join(":"))                
            end
        end
    }
    sortedEvents={}
    noteEvents.each{|k,v|
        s=v.sort{|x,y|;x[0]<=>y[0]}
        sortedEvents[k]=s
    }
    return sortedEvents
end

def parseScore(lines)
    midiEvents={}
    lines.each_with_index{|l,i|
        if(l.include?("<Absolute>"))
            tick=l.split(/[><]/)[2].to_i
tick=tick/10
            event=lines[i+1]
            #event='<NoteOn Channel="10" Note="46" Velocity="0"/>'
            #clear out the cruft.
            event.gsub!(/[><\/\"]/,"")
            parts=event.split(" ")
            #puts parts
            if(parts[0]=="NoteOn")
                channel=parts[1].split("=")[1].to_i    
                note=parts[2].split("=")[1].to_i    
                velocity=parts[3].split("=")[1].to_i
                midiEvents[tick]=[] if(midiEvents[tick]==nil) 
                midiEvents[tick].push([parts[0],channel,note,velocity])
                #puts([tick,parts[0],channel,note,velocity].join(":"))                
            end
        end
    }
    return midiEvents
end



module SPMIDI_old


if(PLATFORM=="i386-mswin32")
    extend DL::Importable
    dlload("winmm.dll")
    
    extern "int midiOutOpen(void*,int,int,int,int)"
    extern "int midiOutShortMsg(void*,int)"
    extern "int midiOutClose(void*)"
    
    #m=SPMIDI::MidiInterface.new(55)
    class MidiInterface
        def initialize(instrument)
            puts "New Midi interface"
            dat=[1230].pack('L').to_ptr
            puts "Opening.."
            puts SPMIDI::midiOutOpen(dat,-1,0,0,0)
            ptr=dat.to_a('L',1)[0]
            puts ptr
            @hMidi=DL::PtrData.new(ptr)
            setInstrument(0,instrument)
        end
        def stop()
            SPMIDI::midiOutClose(DL::PtrData.new(@hMidi));
        end
        def setInstrument(channel,instrument)
            #puts "set"
            SPMIDI::midiOutShortMsg(@hMidi,(256*instrument)+192+channel);
        end
        
        def startNote(note,velocity=99,channel=0)
            SPMIDI::midiOutShortMsg(@hMidi,(velocity*256+note)*256+144+channel);
        end
        def stopNote(note,velocity=99,channel=0)
            SPMIDI::midiOutShortMsg(@hMidi,(velocity*256+note)*256+128+channel);
        end
    end
    
end
end#module

class MIDIator::Interface
    def setInstrument(channel,instrument)
        #puts "set"
        @driver.program_change( channel, instrument )
        #SPMIDI::midiOutShortMsg(@hMidi,(256*instrument)+192+channel);
    end
    
    def startNote(note,velocity=99,channel=0)
        #SPMIDI::midiOutShortMsg(@hMidi,(velocity*256+note)*256+144+channel);
        @driver.note_on(note,channel,velocity)
    end
    def stopNote(note,velocity=99,channel=0)
        @driver.note_off(note,channel,velocity)
        #SPMIDI::midiOutShortMsg(@hMidi,(velocity*256+note)*256+128+channel);
    end
    def stopAll()
        0.upto(16){|c|
            control_change( 0x7f, c, 0 )
        }
    end
end

def midiInterface()
    if $spMidiInterface==nil
        $spMidiInterface=MIDIator::Interface.new 
        $spMidiInterface.autodetect_driver
    end
    return $spMidiInterface
end
